--[[
More Blocks: slab definitions

Copyright (c) 2011-2017 Hugo Locurcio and contributors.
Licensed under the zlib license. See LICENSE.md for more information.
--]]

local S = moreblocks.intllib
local nodelight = ocutil.nodename_light_source

local enable_moreblocks_old_place =
    minetest.setting_getbool ("enable_moreblocks_old_place")

-- Node will be called <modname>:slab_<subname>

function register_slab(modname, subname, recipeitem, groups, images, description, drop, light)
    stairsplus:register_slab(modname, subname, recipeitem, {
        groups = groups,
        tiles = images,
        description = description,
        drop = drop,
        light_source = light,
        sounds = default.node_sound_stone_defaults(),
    })
end

local slabs_defs = {
    [""] = 8,
    ["_quarter"] = 4,
    ["_three_quarter"] = 12,
    ["_1"] = 1,
    ["_2"] = 2,
    ["_14"] = 14,
    ["_15"] = 15,
    ["_two_sides"] = {
                        { -0.5, -0.5, -0.5, 0.5, -7/16, 7/16 },
                        { -0.5, -0.5, 7/16, 0.5, 0.5, 0.5 }
                    },
    ["_three_sides"] = {
                        { -7/16, -0.5, -0.5, 0.5, -7/16, 7/16 },
                        { -7/16, -0.5, 7/16, 0.5, 0.5, 0.5 },
                        { -0.5, -0.5, -0.5, -7/16, 0.5, 0.5 }
                    },
    ["_three_sides_u"] = {
                        { -0.5, -0.5, -0.5, 0.5, 0.5, -7/16 },
                        { -0.5, -0.5, -7/16, 0.5, -7/16, 7/16 },
                        { -0.5, -0.5, 7/16, 0.5, 0.5, 0.5 }
                    }
}

for k,v in pairs(slabs_defs) do
    table.insert(stairsplus.shapes_list, { "slab_", k })
end

function stairsplus:register_slab_alias(modname_old, subname_old, modname_new, subname_new)
    local defs = stairsplus.copytable(slabs_defs)
    for alternate, def in pairs(defs) do
        minetest.register_alias(modname_old .. ":slab_" .. subname_old .. alternate, modname_new .. ":slab_" .. subname_new .. alternate)
    end
end

function stairsplus:register_slab_alias_force(modname_old, subname_old, modname_new, subname_new)
    local defs = stairsplus.copytable(slabs_defs)
    for alternate, def in pairs(defs) do
        minetest.register_alias_force(modname_old .. ":slab_" .. subname_old .. alternate, modname_new .. ":slab_" .. subname_new .. alternate)
    end
end

function stairsplus:register_slab(modname, subname, recipeitem, fields)
    local defs = stairsplus.copytable(slabs_defs)
    local desc_base = S("%s Slab"):format(fields.description)
    for alternate, shape in pairs(defs) do

        local def = {}

        if type(shape) ~= "table" then
            def = {
                    node_box = {
                        type = "fixed",
                        fixed = {-0.5, -0.5, -0.5, 0.5, (shape/16)-0.5, 0.5},
                    },
                    description = ("%s (%d/16)"):format(desc_base, shape)
                }
        else
            def = {
                node_box = {
                    type = "fixed",
                    fixed = shape,
                },
                description = desc_base
            }
        end

        for k, v in pairs(fields) do
            def[k] = v
        end

        def.drawtype   = "nodebox"
        def.paramtype  = "light"
        def.paramtype2 = def.paramtype2 or "facedir"

        if enable_moreblocks_old_place then
            def.on_place   = minetest.rotate_node
        else
            def.on_place   = function (itemstack, placer, pointed_thing)
                if pointed_thing.type ~= "node" then
                    return itemstack
                end

                -- If it's being placed on an another similar one,
                -- replace it with a full block
                local slabpos  = nil
                local slabnode = nil
                local p0 = pointed_thing.under
                local p1 = pointed_thing.above
                local n0 = minetest.get_node(p0)
                local n1 = minetest.get_node(p1)
                local param2 = 0

                local n0_is_upside_down = (n0.name == "moreblocks:slab_"
                    .. subname and n0.param2 >= 20)

                if n0.name == "moreblocks:slab_" .. subname and
                    not n0_is_upside_down and
                    p0.y + 1 == p1.y then
                    slabpos  = p0
                    slabnode = n0
                elseif n1.name == "moreblocks:slab_" .. subname then
                    slabpos = p1
                    slabnode = n1
                end

                if slabpos then
                    -- Remove the slab at slabpos
                    minetest.remove_node (slabpos)
                    -- Make a fake stack of a single item and
                    -- try to place it
                    local fakestack = ItemStack (recipeitem)
                    fakestack:set_count(itemstack:get_count())

                    pointed_thing.above = slabpos
                    local success
                    fakestack, success = minetest.item_place (fakestack,
                        placer, pointed_thing)
                    -- If the item was taken from the fake stack,
                    -- decrement original
                    if success then
                        itemstack:set_count(fakestack:get_count())
                    -- Else put old node back
                    else
                        minetest.set_node (slabpos, slabnode)
                    end
                    return itemstack
                end

                -- Upside down slabs
                if p0.y - 1 == p1.y then
                    -- Turn into full block if pointing at a existing slab
                    if n0_is_upside_down  then
                        -- Remove the slab at the position of the slab
                        minetest.remove_node (p0)
                        -- Make a fake stack of a single item and try to place it
                        local fakestack = ItemStack(recipeitem)
                        fakestack:set_count(itemstack:get_count())

                        pointed_thing.above = p0
                        local success
                        fakestack, success = minetest.item_place (fakestack,
                            placer, pointed_thing)
                        -- If the item was taken from the fake
                        -- stack, decrement original
                        if success then
                            itemstack:set_count(fakestack:get_count())
                        -- Else put old node back
                        else
                            minetest.set_node (p0, n0)
                        end
                        return itemstack
                    end

                    -- Place upside down slab
                    param2 = 20
                end

                -- If pointing at the side of a upside down slab
                if n0_is_upside_down and p0.y + 1 ~= p1.y then
                    param2 = 20
                end

                return minetest.item_place (itemstack,
                    placer, pointed_thing, param2)
            end
        end

        def.groups = stairsplus:prepare_groups(fields.groups)
        if fields.drop and not (type(fields.drop) == "table") then
            def.drop = modname.. ":slab_" .. fields.drop .. alternate
        end

        if  def.light_source == nil then
            def.light_source =  nodelight (recipeitem)
        end

        minetest.register_node(":" .. modname .. ":slab_" .. subname .. alternate, def)
    end
--  minetest.register_alias("stairs:slab_" .. subname, modname .. ":slab_" .. subname)

    circular_saw.known_nodes[recipeitem] = {modname, subname}

    -- Some saw-less recipes:

-- RJK 2019: Slabs and stairs of "moreblocks:" nodes  belong to "more-
-- blocks:". Slabs and stairs of anything else belong to "stairs:"  as
-- long as they exist in "stairs:".

        local wantname = modname

        if ocutil.not_starts_with (recipeitem, "moreblocks:") then
            local maybe_node =
                "stairs:slab_" .. modname .. "_" .. subname

            if  minetest.registered_nodes [maybe_node] ~= nil then
                wantname = "stairs"
            end
        end

        if wantname ~= "stairs" then
            minetest.register_craft({
                output = wantname .. ":slab_" .. subname .. " 6",
                recipe = {{recipeitem, recipeitem, recipeitem}},
            })
        end

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname,
            recipe = {modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname},
        })

        -- uncomment this rule when conflict is no longer likely to happen
        --  https://github.com/minetest/minetest/issues/2881
        -- minetest.register_craft({
        --  type = "shapeless",
        --  output = modname .. ":slab_" .. subname,
        --  recipe = {modname .. ":panel_" .. subname, modname .. ":panel_" .. subname},
        -- })

        -- then remove these two
        minetest.register_craft({
            output = modname .. ":slab_" .. subname,
            recipe = {{modname .. ":panel_" .. subname, modname .. ":panel_" .. subname}},
        })

        minetest.register_craft({
            output = modname .. ":slab_" .. subname,
            recipe = {
                {modname .. ":panel_" .. subname},
                {modname .. ":panel_" .. subname},
            },
        })
        ------------------------------

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname, modname .. ":slab_" .. subname},
        })

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname .. "_three_quarter", modname .. ":slab_" .. subname .. "_quarter"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname .. "_14", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = recipeitem,
            recipe = {modname .. ":slab_" .. subname .. "_15", modname .. ":slab_" .. subname .. "_1"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname,
            recipe = {modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname,
            recipe = {modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname,
            recipe = {modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_quarter",
            recipe = {modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_quarter",
            recipe = {modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_2",
            recipe = {modname .. ":slab_" .. subname .. "_1", modname .. ":slab_" .. subname .. "_1"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_three_quarter",
            recipe = {modname .. ":slab_" .. subname, modname .. ":slab_" .. subname .. "_quarter"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_three_quarter",
            recipe = {modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter", modname .. ":slab_" .. subname .. "_quarter"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_three_quarter",
            recipe = {modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_14",
            recipe = {modname .. ":slab_" .. subname .. "_three_quarter", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_14",
            recipe = {modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2", modname .. ":slab_" .. subname .. "_2"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. "_15",
            recipe = {modname .. ":slab_" .. subname .. "_14", modname .. ":slab_" .. subname .. "_1"},
        })

        minetest.register_craft({
            type = "shapeless",
            output = modname .. ":slab_" .. subname .. " 3",
            recipe = {modname .. ":stair_" .. subname, modname .. ":stair_" .. subname},
        })
end
