-- ===================================================================

local reg_alias = ocutil.safe_register_alias

local enable_slow_leaves   =
      minetest.setting_getbool ("enable_slow_leaves"   )

local dtleaves = "glasslike"
if enable_slow_leaves then
      dtleaves = "allfaces_optional"
end

local                            winter_wonderland     =
      minetest.setting_getbool ("winter_wonderland"    )

local node_name
local img

-- ===================================================================
-- TBD: Review and document this part.

minetest.register_node ("default:emerge", {
    description = "Emerge",
    tiles = {"default_stone.png"} ,
    groups = {cracky = 3, stone = 1} ,
    drop = 'default:cobble',
    legacy_mineral = true,
    sounds = default.sound_stone() ,
    workbench = true ,
})
minetest.register_alias ("emerge", "default:emerge")

-- ===================================================================
-- Stone

minetest.register_node ("default:stone", {
    description = "Stone",
    tiles = {"default_stone.png"} ,
    groups = {cracky = 3, stone = 1} ,
    drop = 'default:cobble',
    legacy_mineral = true,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:lumistone", {
    description    = "Luminescent Stone"         ,
    tiles          = { "default_lumistone.png" } ,
    groups         = { cracky = 3, stone = 1   } ,
    drop           = "default:cobble"            ,
    legacy_mineral = true                        ,
    light_source   = 6                           ,
    paramtype      = "light"                     ,

    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:flooddiv", {
    description = "Flood barrier",
    tiles = { "default_stone.png" } ,
    is_ground_content = true,
    groups = {cracky=3} ,
    drop = 'default:cobble',
    legacy_mineral = true,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:cobble", {
    description = "Cobblestone",
    tiles = {"default_cobble.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, stone = 2} ,
    workbench = true ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:stonebrick", {
    description = "Stone Brick",
    tiles = {"default_stone_brick.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, stone = 1} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_alias ("default:stone_brick" ,
                         "default:stonebrick"  )

-- ===================================================================

minetest.register_node ("default:mossycobble", {
    description = "Mossy Cobblestone",
    tiles = {"default_mossycobble.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, stone = 1} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:desert_stone", {
    description = "Desert Stone",
    tiles = {"default_desert_stone.png"} ,
    groups = {cracky = 3, stone = 1} ,
    drop = 'default:desert_cobble',
    legacy_mineral = true,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:desert_cobble", {
    description = "Desert Cobblestone",
    tiles = {"default_desert_cobble.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, stone = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:desert_stonebrick", {
    description = "Desert Stone Brick",
    tiles = {"default_desert_stone_brick.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, stone = 1} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:sandstone", {
    description = "Sandstone",
    tiles = {"default_sandstone.png"} ,
    groups = {crumbly = 1, cracky = 3} ,
    workbench = true ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:sandstonebrick", {
    description = "Sandstone Brick",
    tiles = {"default_sandstone_brick.png"} ,
    is_ground_content = false,
    groups = {cracky = 3} ,
    workbench = true ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:obsidian", {
    description = "Obsidian",
    tiles = {"default_obsidian.png"} ,
    sounds = default.sound_stone() ,
    groups = {cracky = 3, level = 2} ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:obsidianbrick", {
    description = "Obsidian Brick",
    tiles = {"default_obsidian_brick.png"} ,
    is_ground_content = false,
    sounds = default.sound_stone() ,
    groups = {cracky = 3, level = 2} ,
})

-- ===================================================================

local WORM_CHANCE = 80

local dig_worm = function (pos, oldnode, oldmetadata, digger)
    if true then return end
    if math.random (1, 100) > WORM_CHANCE then return end
    local tool = digger:get_wielded_item():get_name()

    if  ocutil.str_empty (tool) or
        ocutil.starts_with (tool, "default:dirt") then

        minetest.add_entity ({
            x = pos.x, y = pos.y+0.4, z = pos.z
        }, "default:earthworm_entity")
    end
end

-- ===================================================================
-- Soft / Non-Stone

ocutil.register_node_bucketable ("default:dirt", {
    description = "Dirt" ,
    bucket_base = "dirt" ,
    tiles = {"default_dirt.png"} ,
    groups = { crumbly=3, soil=1 } ,
    sounds = default.node_sound_dirt_defaults() ,
    after_dig_node = dig_worm ,
})

-- ===================================================================

local dwg_tiles = {
    "default_grass.png" , "default_dirt.png" ,
    { name = "default_dirt.png^default_grass_side.png" ,
      tileable_vertical = false }
}

if winter_wonderland then
    dwg_tiles = {
        "default_snow.png" , "default_dirt.png" ,
        { name = "default_dirt.png^default_snow.png" ,
          tileable_vertical = false }
    }
end

ocutil.register_node_bucketable ("default:dirt_with_grass", {
    description = "Dirt with Grass" ,
    bucket_base = "dirt_with_grass" ,
    tiles       = dwg_tiles ,
    groups      = { crumbly = 3, soil = 1 } ,
    drop        = 'default:dirt',

    sounds      = default.node_sound_dirt_defaults ({
        footstep = {name = "default_grass_footstep", gain = 0.25} ,
    }) ,

    after_dig_node = dig_worm ,
})

-- ===================================================================

minetest.register_node ("default:dirt_with_grass_footsteps", {
    description = "Dirt with Grass and Footsteps",
    tiles = {"default_grass.png^default_footprint.png", "default_dirt.png",
        {name = "default_dirt.png^default_grass_side.png",
            tileable_vertical = false}} ,
    groups = {crumbly = 3, soil = 1, not_in_creative_inventory = 1} ,
    drop = 'default:dirt',
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.25} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:dirt_with_dry_grass", {
    description = "Dirt with Dry Grass",
    tiles = {"default_dry_grass.png",
        "default_dirt.png",
        {name = "default_dirt.png^default_dry_grass_side.png",
            tileable_vertical = false}} ,
    groups = {crumbly = 3, soil = 1} ,
    drop = 'default:dirt',
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.4} ,
    }),

    after_dig_node = dig_worm ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:dirt_with_snow", {

    bucket_base = "dirt_with_snow" ,
    description = "Dirt with Snow" ,
    tiles       = {
        "default_snow.png" ,
        "default_dirt.png" ,
        {
            name = "default_dirt.png^default_snow_side.png" ,
            tileable_vertical = false
        }
    } ,

    groups = {crumbly = 3, soil = 1} ,
    drop = 'default:dirt',
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:dirt_with_rainforest_litter", {

    description = "Dirt with Rainforest Litter",
    tiles = {
        "default_rainforest_litter.png",
        "default_dirt.png",
        {name = "default_dirt.png^default_rainforest_litter_side.png",
            tileable_vertical = false}
    } ,
    groups = {crumbly = 3, soil = 1, spreading_dirt_type = 1} ,
    drop = "default:dirt",
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.4} ,
    }),

    after_dig_node = dig_worm ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:sand", {

    bucket_base = "sand"                                ,
    description = "Sand"                                ,
    tiles       = { "default_sand.png" }                ,
    groups      = { crumbly=3, falling_node=1, sand=1 } ,
    sounds      = default.node_sound_sand_defaults()    ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:desert_sand", {

    bucket_base = "desert_sand"                         ,
    description = "Desert Sand"                         ,
    tiles       = { "default_desert_sand.png" }         ,
    groups      = { crumbly=3, falling_node=1, sand=1 } ,
    sounds      = default.node_sound_sand_defaults()    ,
})

-- ===================================================================

minetest.register_node ("default:desert_sandstone", {

    description = "Desert Sandstone",
    tiles = {"default_desert_sandstone.png"} ,
    groups = {crumbly = 1, cracky = 3} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:desert_sandstone_brick", {

    description = "Desert Sandstone Brick",
    paramtype2 = "facedir",
    place_param2 = 0,
    tiles = {"default_desert_sandstone_brick.png"} ,
    is_ground_content = false,
    groups = {cracky = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:desert_sandstone_block", {

    description = "Desert Sandstone Block",
    tiles = {"default_desert_sandstone_block.png"} ,
    is_ground_content = false,
    groups = {cracky = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:stone_block", {

    description = "Stone Block",
    tiles = {"default_stone_block.png"} ,
    is_ground_content = false,
    groups = {cracky = 2, stone = 1} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:desert_stone_block", {

    description = "Desert Stone Block",
    tiles = {"default_desert_stone_block.png"} ,
    is_ground_content = false,
    groups = {cracky = 2, stone = 1} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:sandstone_block", {

    description = "Sandstone Block",
    tiles = {"default_sandstone_block.png"} ,
    is_ground_content = false,
    groups = {cracky = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:obsidian_block", {

    description = "Obsidian Block",
    tiles = {"default_obsidian_block.png"} ,
    is_ground_content = false,
    sounds = default.sound_stone() ,
    groups = {cracky = 1, level = 2} ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:silver_sandstone_block", {

    description = "Silver Sandstone Block",
    tiles = {"default_silver_sandstone_block.png"} ,
    is_ground_content = false,
    groups = {cracky = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:silver_sandstone", {

    description = "Silver Sandstone",
    tiles = {"default_silver_sandstone.png"} ,
    groups = {crumbly = 1, cracky = 3} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:silver_sandstone_brick", {
    description = "Silver Sandstone Brick",
    paramtype2 = "facedir",
    place_param2 = 0,
    tiles = {"default_silver_sandstone_brick.png"} ,
    is_ground_content = false,
    groups = {cracky = 2} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:silver_sand", {

    bucket_base = "silver_sand"                         ,
    description = "Silver Sand"                         ,
    tiles       = { "default_silver_sand.png" }         ,
    groups      = { crumbly=3, falling_node=1, sand=1 } ,
    sounds      = default.node_sound_sand_defaults()    ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:gravel", {

    bucket_base = "gravel"                             ,
    description = "Gravel"                             ,
    tiles       = { "default_gravel.png" }             ,
    groups      = { crumbly = 2, falling_node = 1 }    ,
    sounds      = default.node_sound_gravel_defaults() ,

    drop = {
        max_items = 1,
        items = {
            {items = {'default:flint'} , rarity = 16} ,
            {items = {'default:gravel'}}
        }
    }
})

-- ===================================================================

ocutil.register_node_bucketable ("default:clay", {

    bucket_base = "clay" ,
    description = "Clay" ,
    tiles = {"default_clay.png"} ,
    groups = {crumbly = 3} ,
    drop = 'default:clay_lump 4',
    sounds = default.node_sound_dirt_defaults() ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:snow", {

    bucket_base = "snow" ,
    description = "Snow" ,
    tiles = {"default_snow.png"} ,
    inventory_image = "default_snowball.png",
    wield_image = "default_snowball.png",
    paramtype = "light",
    buildable_to = true,
    floodable = true,
    drawtype = "nodebox",

    node_box = {
        type = "fixed",
        fixed = {
            {-0.5, -0.5, -0.5, 0.5, -0.25, 0.5} ,
        } ,
    } ,

    groups = {crumbly = 3, falling_node = 1, puts_out_fire = 1} ,
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
        dug = {name = "default_snow_footstep", gain = 0.2} ,
        dig = {name = "default_snow_footstep", gain = 0.2}
    }) ,

    on_construct = function (pos)
        pos.y = pos.y - 1
        if  minetest.get_node (pos).name ==
                "default:dirt_with_grass" then
            minetest.set_node (pos,
                { name = "default:dirt_with_snow" })
        end
    end ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:snowblock", {

    bucket_base = "snow_block" ,
    description = "Snow Block" ,
    tiles = {"default_snow.png"} ,
    groups = {crumbly = 3, puts_out_fire = 1} ,
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
        dug = {name = "default_snow_footstep", gain = 0.2} ,
        dig = {name = "default_snow_footstep", gain = 0.2}
    }),
})

-- ===================================================================

-- 'is ground content = false' to avoid tunnels in sea ice or ice rivers

ocutil.register_node_bucketable ("default:ice", {
    description = "Ice" ,
    bucket_base = "ice" ,
    is_ground_content = false,
    paramtype = "light",
    tiles     = { "default_ice.png" } ,

    groups = {
        cracky=3     , puts_out_fire=1  ,
        cools_lava=1 , slippery=3       ,
    } ,

    sounds = default.node_sound_glass_defaults() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:ice_nine", {
    description = "Ice Nine Beware",
    is_ground_content = false,
    paramtype = "light",
    tiles             = { "default_ice.png" } ,

    groups = {
        cracky=3     , puts_out_fire=1  ,
        cools_lava=1 , slippery=3       ,
        not_in_creative_inventory=1
    } ,

    sounds = default.node_sound_glass_defaults() ,
})

-- ===================================================================

minetest.register_abm ({
    nodenames = {
        "codersea:water_source"   ,
        "codersea:water_flowing"  ,
        "default:water_source"    ,
        "default:water_flowing"   ,
    } ,

    neighbors = { "default:ice_nine" } ,
    interval  =  5        ,
    chance    =  1        ,
    catch_up  = false     ,

    action    = function (pos, node)
        minetest.set_node (pos, { name = "default:ice_nine" })
    end ,
})

-- ===================================================================

-- Mapgen-placed ice with 'is ground content = true' to contain tunnels

minetest.register_node ("default:cave_ice", {
    description = "Cave Ice"            ,
    drop        = "default:ice"         ,
    tiles       = { "default_ice.png" } ,
    paramtype   = "light"               ,

    groups = {
        cracky=3     , puts_out_fire=1 ,
        cools_lava=1 , slippery=3      ,
        not_in_creative_inventory=1    ,
    } ,

    sounds = default.node_sound_glass_defaults() ,
})

-- ===================================================================
-- Related to trees but separate

ocutil.register_node_bucketable ("default:wood", {
    description = "Wooden Planks",
    tiles = {"default_wood.png"} ,
    bucket_base = "wood" ,
    is_ground_content = false,
    workbench = true ,
    groups = { choppy = 2, oddly_breakable_by_hand = 2, flammable = 3, wood = 1} ,
    sounds = default.sound_wood() ,
})

-- ===================================================================

minetest.register_node ("default:sapling", {
    description = "Sapling",
    drawtype = "plantlike",
    visual_scale = 1.0,
    tiles = {"default_sapling.png"} ,
    inventory_image = "default_sapling.png",
    wield_image = "default_sapling.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    selection_box = {
        type = "fixed",
        fixed = {-0.3, -0.5, -0.3, 0.3, 0.35, 0.3}
    } ,
    groups = {snappy = 2, dig_immediate = 3, flammable = 2,
        attached_node = 1, sapling = 1} ,
    sounds = default.sound_leaves() ,
})

-- ===================================================================

minetest.register_node ("default:apple", {
    description = "Apple",
    drawtype = "plantlike",
    visual_scale = 1.0,
    tiles = {"default_apple.png"} ,
    inventory_image = "default_apple.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    is_ground_content = false,
    selection_box = {
        type = "fixed",
        fixed = {-0.2, -0.5, -0.2, 0.2, 0, 0.2}
    } ,
    groups = {fleshy = 3, dig_immediate = 3, flammable = 2,
        leafdecay = 3, leafdecay_drop = 1} ,
    on_use = minetest.item_eat(2),
    sounds = default.sound_leaves() ,

    after_place_node = function (pos, placer, itemstack)
        if placer:is_player() then
            minetest.set_node (pos, {name = "default:apple", param2 = 1})
        end
    end ,
})

-- ===================================================================

minetest.register_node ("default:leaves", {
    description = "Leaves",
    drawtype = dtleaves,
    waving = 1,
    visual_scale = 1.3,
    tiles = {"default_leaves.png"} ,
    special_tiles = {"default_leaves_simple.png"} ,
    paramtype = "light",
    is_ground_content = false,
    groups = {snappy = 3, leafdecay = 3, flammable = 2, leaves = 1} ,
    drop = {
        max_items = 1,
        items = {
            {
                -- player will get sapling with 1/20 chance
                items = {'default:sapling'} ,
                rarity = 20,
            } ,
            {
                -- player will get leaves only if he get no saplings,
                -- this is because max_items is 1
                items = {'default:leaves'} ,
            }
        }
    } ,

    sounds = default.sound_leaves() ,
})

-- ===================================================================
-- Bushes and/or related

minetest.register_node ("default:bush_leaves", {
    description = "Bush Leaves",
    drawtype = dtleaves,
    waving = 1,
    tiles = {"default_leaves_simple.png"} ,
    paramtype = "light",
    groups = {snappy = 3, flammable = 2, leaves = 1} ,
    drop = {
        max_items = 1,
        items = {
--          {items = {"default:bush_sapling"} , rarity = 5} ,
            {items = {"default:bush_leaves"}}
        }
    } ,

    sounds = default.sound_leaves() ,
    after_place_node = default.after_place_leaves,
})

-- ===================================================================

minetest.register_node ("default:bush_stem", {
    description = "Bush Stem",
    drawtype = "plantlike",
    visual_scale = 1.41,
    tiles = {"default_bush_stem.png"} ,
    inventory_image = "default_bush_stem.png",
    wield_image = "default_bush_stem.png",
    paramtype = "light",
    sunlight_propagates = true,
    groups = {choppy = 2, oddly_breakable_by_hand = 1, flammable = 2} ,
    sounds = default.sound_wood() ,
    selection_box = {
        type = "fixed",
        fixed = {-7 / 16, -0.5, -7 / 16, 7 / 16, 0.5, 7 / 16} ,
    } ,
})

-- ===================================================================

minetest.register_node ("default:acacia_bush_stem", {
    description = "Acacia Bush Stem",
    drawtype = "plantlike",
    visual_scale = 1.41,
    tiles = {"default_acacia_bush_stem.png"} ,
    inventory_image = "default_acacia_bush_stem.png",
    wield_image = "default_acacia_bush_stem.png",
    paramtype = "light",
    sunlight_propagates = true,
    groups = {choppy = 2, oddly_breakable_by_hand = 1, flammable = 2} ,
    sounds = default.sound_wood() ,
    selection_box = {
        type = "fixed",
        fixed = {-7 / 16, -0.5, -7 / 16, 7 / 16, 0.5, 7 / 16} ,
    } ,
})

-- ===================================================================

minetest.register_node ("default:acacia_bush_leaves", {
    description = "Acacia Bush Leaves",
    drawtype = dtleaves,
    waving = 1,
    tiles = {"default_acacia_leaves_simple.png"} ,
    paramtype = "light",
    groups = {snappy = 3, flammable = 2, leaves = 1} ,
    drop = {
        max_items = 1,
        items = {
--          {items = {"default:acacia_bush_sapling"} , rarity = 5} ,
            {items = {"default:acacia_bush_leaves"}}
        }
    } ,
    sounds = default.sound_leaves() ,

    after_place_node = default.after_place_leaves,
})

-- ===================================================================
-- Ores

minetest.register_node ("default:stone_with_coal", {
    description = "Coal Ore",
    tiles = {"default_stone.png^default_mineral_coal.png"} ,
    groups = {cracky = 3} ,
    drop = 'default:coal_lump',
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:coalblock", {
    description = "Coal Block",
    tiles = {"default_coalblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 3} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:stone_with_iron", {
    description = "Iron Ore",
    tiles = {"default_stone.png^default_mineral_iron.png"} ,
    groups = {cracky = 3} ,
    drop = 'default:iron_lump',
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:steelblock", {
    description = "Steel Block",
    tiles = {"default_steelblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, level = 2} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:stone_with_copper", {
    description = "Copper Ore",
    tiles = {"default_stone.png^default_mineral_copper.png"} ,
    groups = {cracky = 3} ,
    drop = 'default:copper_lump',
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:copperblock", {
    description = "Copper Block",
    tiles = {"default_copperblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, level = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:stone_with_tin", {
    description = "Tin Ore",
    tiles = {"default_stone.png^default_mineral_tin.png"} ,
    groups = {cracky = 2} ,
    drop = "default:tin_lump",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:tinblock", {
    description = "Tin Block",
    tiles = {"default_tinblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 1, level = 2} ,
    sounds = default.node_sound_metal_defaults() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:bronzeblock", {
    description = "Bronze Block",
    tiles = {"default_bronzeblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 3, level = 2} ,
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:stone_with_mese", {
    description = "Mese Ore",
    tiles = {"default_stone.png^default_mineral_mese.png"} ,
    groups = {cracky = 3} ,
    drop = "default:mese_crystal",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:mese", {
    description = "Mese Block",
    tiles = {"default_mese.png"} ,
    paramtype = "light",
    groups = {cracky = 3, level = 2} ,
    sounds = default.sound_stone() ,
    light_source = 3,
})

-- ===================================================================

minetest.register_node ("default:stone_with_gold", {
    description = "Gold Ore",
    tiles = {"default_stone.png^default_mineral_gold.png"} ,
    groups = {cracky = 3} ,
    drop = "default:gold_lump",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:goldblock", {
    description = "Gold Block",
    tiles = {"default_goldblock.png"} ,
    is_ground_content = false,
    groups = {cracky = 3} ,
    sounds = default.sound_stone() ,
    paramtype = "light",
    light_source = 3 ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:stone_with_diamond", {
    description = "Diamond Ore",
    tiles = {"default_stone.png^default_mineral_diamond.png"} ,
    groups = {cracky = 3} ,
    drop = "default:diamond",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:diamondblock", {
    description = "Diamond Block",
    tiles = {"default_diamondblock.png"} ,
    is_ground_content = false,
    groups    = { cracky = 3, level = 3 } ,
    sounds    = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================
-- Plantlife (non-cubic)

minetest.register_node ("default:cactus", {
    description = "Cactus",
    tiles = {"default_cactus_top.png", "default_cactus_top.png",
        "default_cactus_side.png"} ,
    paramtype2 = "facedir",
    groups = {snappy = 1, choppy = 3} ,
    sounds = default.sound_wood() ,
    on_place = minetest.rotate_node,
})

-- ===================================================================

minetest.register_node ("default:papyrus", {
    description = "Papyrus",
    drawtype = "plantlike",
    tiles = {"default_papyrus.png"} ,
    inventory_image = "default_papyrus.png",
    wield_image = "default_papyrus.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    selection_box = {
        type = "fixed",
        fixed = {-0.3, -0.5, -0.3, 0.3, 0.5, 0.3}
    } ,
    groups = {snappy = 3, flammable = 2} ,
    sounds = default.sound_leaves() ,

    after_dig_node = function (pos, node, metadata, digger)
        default.dig_up(pos, node, digger)
    end ,
})

-- ===================================================================

minetest.register_node ("default:dry_shrub", {
    description = "Dry Shrub",
    drawtype = "plantlike",
    waving = 1,
    visual_scale = 1.0,
    tiles = {"default_dry_shrub.png"} ,
    inventory_image = "default_dry_shrub.png",
    wield_image = "default_dry_shrub.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,
    groups = {snappy = 3, flammable = 3, attached_node = 1} ,
    sounds = default.sound_leaves() ,
    selection_box = {
        type = "fixed",
        fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
    } ,
})

-- ===================================================================

minetest.register_node ("default:junglegrass", {
    description = "Jungle Grass",
    drawtype = "plantlike",
    waving = 1,
    visual_scale = 1.3,
    tiles = {"default_junglegrass.png"} ,
    inventory_image = "default_junglegrass.png",
    wield_image = "default_junglegrass.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,
    groups = {snappy = 3, flora = 1, attached_node = 1, grass = 1} ,
    sounds = default.sound_leaves() ,
    selection_box = {
        type = "fixed",
        fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
    } ,
})

-- ===================================================================

minetest.register_node ("default:grass_1", {
    description = "Grass",
    drawtype = "plantlike",
    waving = 1,
    tiles = {"default_grass_1.png"} ,
    -- Use texture of a taller grass stage in inventory
    inventory_image = "default_grass_3.png",
    wield_image = "default_grass_3.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,
    groups = {snappy = 3, flora = 1, attached_node = 1, grass = 1} ,
    sounds = default.sound_leaves() ,
    selection_box = {
        type = "fixed",
        fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
    } ,

    on_place = function (itemstack, placer, pointed_thing)
        -- place a random grass node
        local stack = ItemStack("default:grass_" .. math.random(1,5))
        local ret = minetest.item_place(stack, placer, pointed_thing)
        return ItemStack("default:grass_1 " ..
            itemstack:get_count() - (1 - ret:get_count()))
    end ,
})

-- ===================================================================

for i = 2, 5 do
    minetest.register_node ("default:grass_" .. i, {
        description = "Grass",
        drawtype = "plantlike",
        waving = 1,
        tiles = {"default_grass_" .. i .. ".png"} ,
        inventory_image = "default_grass_" .. i .. ".png",
        wield_image = "default_grass_" .. i .. ".png",
        paramtype = "light",
        sunlight_propagates = true,
        walkable = false,
        buildable_to = true,
        drop = "default:grass_1",
        groups = {snappy = 3, flora = 1, attached_node = 1,
            not_in_creative_inventory = 1, grass = 1} ,
        sounds = default.sound_leaves() ,
        selection_box = {
            type = "fixed",
            fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
        } ,
    })
end

-- ===================================================================

minetest.register_node ("default:dry_grass_1", {
    description = "Dry Grass",
    drawtype = "plantlike",
    waving = 1,
    tiles = {"default_dry_grass_1.png"} ,
    inventory_image = "default_dry_grass_3.png",
    wield_image = "default_dry_grass_3.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,
    groups = {snappy = 3, flammable = 3, flora = 1,
        attached_node = 1, dry_grass = 1} ,
    sounds = default.sound_leaves() ,
    selection_box = {
        type = "fixed",
        fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
    } ,

    on_place = function (itemstack, placer, pointed_thing)
        -- place a random dry grass node
        local stack = ItemStack("default:dry_grass_" .. math.random(1, 5))
        local ret = minetest.item_place(stack, placer, pointed_thing)
        return ItemStack("default:dry_grass_1 " ..
            itemstack:get_count() - (1 - ret:get_count()))
    end ,
})

-- ===================================================================

for i = 2, 5 do
    minetest.register_node ("default:dry_grass_" .. i, {
        description = "Dry Grass",
        drawtype = "plantlike",
        waving = 1,
        tiles = {"default_dry_grass_" .. i .. ".png"} ,
        inventory_image = "default_dry_grass_" .. i .. ".png",
        wield_image = "default_dry_grass_" .. i .. ".png",
        paramtype = "light",
        sunlight_propagates = true,
        walkable = false,
        buildable_to = true,
        groups = {snappy = 3, flammable = 3, flora = 1, attached_node = 1,
            not_in_creative_inventory=1, dry_grass = 1} ,
        drop = "default:dry_grass_1",
        sounds = default.sound_leaves() ,
        selection_box = {
            type = "fixed",
            fixed = {-0.5, -0.5, -0.5, 0.5, -5/16, 0.5} ,
        } ,
    })
end

-- ===================================================================
-- Corals

minetest.register_node ("default:coral_blue", {
    description = "Blue Coral",
    tiles = {"default_coral_blue.png"} ,
    groups = {cracky = 3} ,
    drop = "default:coral_skeleton",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:coral_brown", {
    description = "Brown Coral",
    tiles = {"default_coral_brown.png"} ,
    groups = {cracky = 3} ,
    drop = "default:coral_skeleton",
    sounds = default.sound_stone() ,
})

-- ===================================================================

minetest.register_node ("default:coral_orange", {
    description = "Orange Coral"                    ,
    tiles  = { "default_coral_orange.png" }         ,
    groups = { cracky = 3 }                         ,
    drop   = "default:coral_skeleton"               ,
    sounds = default.sound_stone()    ,
})

-- ===================================================================

minetest.register_node ("default:coral_cyan", {
    description = "Cyan Coral"                    ,
    tiles  = { "default_coral_cyan.png" }         ,
    groups = { cracky = 3 }                         ,
    drop   = "default:coral_skeleton"               ,
    sounds = default.sound_stone()    ,
})

-- ===================================================================

minetest.register_node ("default:coral_pink", {
    description = "Pink Coral"                      ,
    tiles  = { "default_coral_pink.png" }           ,
    groups = { cracky = 3 }                         ,
    drop   = "default:coral_skeleton"               ,
    sounds = default.sound_stone()    ,
})

-- ===================================================================

minetest.register_node ("default:coral_green", {
    description = "Green Coral"                     ,
    tiles  = { "default_coral_green.png" }          ,
    groups = { cracky = 3 }                         ,
    drop   = "default:coral_skeleton"               ,
    sounds = default.sound_stone()    ,
})

-- ===================================================================

minetest.register_node ("default:coral_skeleton", {
    description = "Coral Skeleton",
    tiles = {"default_coral_skeleton.png"} ,
    groups = {cracky = 3} ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ============================================================
-- Tools / "Advanced" crafting / Non-"natural"

minetest.register_node ("default:torch", {
    description = "Torch",
    drawtype = "torchlike",
    tiles = {
        {
            name = "default_torch_on_floor_animated.png",
            animation = {
                type = "vertical_frames",
                aspect_w = 16,
                aspect_h = 16,
                length   =  3.0
            } ,
        } ,
        {
            name="default_torch_on_ceiling_animated.png",
            animation = {
                type = "vertical_frames",
                aspect_w = 16,
                aspect_h = 16,
                length   =  3.0
            } ,
        } ,
        {
            name="default_torch_animated.png",
            animation = {
                type = "vertical_frames",
                aspect_w = 16,
                aspect_h = 16,
                length   =  3.0
            } ,
        } ,
    } ,

    inventory_image = "default_torch_on_floor.png",
    wield_image = "default_torch_on_floor.png",
    paramtype = "light",
    paramtype2 = "wallmounted",
    sunlight_propagates = true,
    is_ground_content = false,
    walkable = false,
    light_source = default.LIGHT_MAX - 1,
    selection_box = {
        type = "wallmounted",
        wall_top = {-0.1, 0.5 - 0.6, -0.1, 0.1, 0.5, 0.1} ,
        wall_bottom = {-0.1, -0.5, -0.1, 0.1, -0.5 + 0.6, 0.1} ,
        wall_side = {-0.5, -0.3, -0.1, -0.5 + 0.3, 0.3, 0.1} ,
    } ,

    groups = {choppy = 2, dig_immediate = 3, flammable = 1, attached_node = 1} ,
    legacy_wallmounted = true,
    sounds = default.node_sound_defaults() ,
})
reg_alias ("torches:floor", "default:torch")

-- ===================================================================

local bookshelf_formspec =
    "size[8,7;]" ..
    default.gui_bg ..
    default.gui_bg_img ..
    default.gui_slots ..
    "list[context;books;0,0.3;8,2;]" ..
    "list[current_player;main;0,2.85;8,1;]" ..
    "list[current_player;main;0,4.08;8,3;8]" ..
    "listring[context;books]" ..
    "listring[current_player;main]" ..
    default.get_hotbar_bg(0,2.85)

-- ===================================================================

minetest.register_node ("default:bookshelf", {
    description = "Bookshelf",
    tiles = {"default_wood.png", "default_wood.png", "default_bookshelf.png"} ,
    is_ground_content = false,
    groups = {choppy = 3, oddly_breakable_by_hand = 2, flammable = 3} ,
    sounds = default.sound_wood() ,

    on_construct = function (pos)
        local meta = minetest.get_meta(pos)
        meta:set_string("formspec", bookshelf_formspec)
        local inv = meta:get_inventory()
        inv:set_size("books", 8 * 2)
    end ,
    can_dig = function (pos,player)
        local inv = minetest.get_meta(pos):get_inventory()
        return inv:is_empty("books")
    end ,
    allow_metadata_inventory_put = function (pos, listname, index, stack)
        if minetest.get_item_group(stack:get_name() , "book") ~= 0 then
            return stack:get_count()
        end
        return 0
    end ,
    on_metadata_inventory_move = function (pos, from_list, from_index, to_list, to_index, count, player)
        minetest.log("action", player:get_player_name() ..
            " moves stuff in bookshelf at " .. minetest.pos_to_string(pos))
    end ,
    on_metadata_inventory_put = function (pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name() ..
            " moves stuff to bookshelf at " .. minetest.pos_to_string(pos))
    end ,
    on_metadata_inventory_take = function (pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name() ..
            " takes stuff from bookshelf at " .. minetest.pos_to_string(pos))
    end ,
    on_blast = function (pos)
        local drops = {}
        default.get_inventory_drops(pos, "books", drops)
        drops[#drops+1] = "default:bookshelf"
        minetest.remove_node (pos)
        return drops
    end ,
})

-- ===================================================================

local function register_sign(material, desc, def)
    minetest.register_node ("default:sign_wall_" .. material, {
        description = desc .. " Sign",
        drawtype = "nodebox",
        tiles = {"default_sign_wall_" .. material .. ".png"} ,
        inventory_image = "default_sign_" .. material .. ".png",
        wield_image = "default_sign_" .. material .. ".png",
        paramtype = "light",
        paramtype2 = "wallmounted",
        sunlight_propagates = true,
        is_ground_content = false,
        walkable = false,
        node_box = {
            type = "wallmounted",
            wall_top    = {-0.4375, 0.4375, -0.3125, 0.4375, 0.5, 0.3125} ,
            wall_bottom = {-0.4375, -0.5, -0.3125, 0.4375, -0.4375, 0.3125} ,
            wall_side   = {-0.5, -0.3125, -0.4375, -0.4375, 0.3125, 0.4375} ,
        } ,
        groups = def.groups,
        legacy_wallmounted = true,
        sounds = def.sounds,

        on_construct = function (pos)
            --local n = minetest.get_node (pos)
            local meta = minetest.get_meta(pos)
            meta:set_string("formspec", "field[text;;${text}]")
            meta:set_string("infotext", "\"\"")
        end ,
        on_receive_fields = function (pos, formname, fields, sender)
            --print("Sign at "..minetest.pos_to_string(pos).." got "..dump(fields))
            local player_name = sender:get_player_name()
            if minetest.is_protected(pos, player_name) then
                minetest.record_protection_violation(pos, player_name)
                return
            end
            local meta = minetest.get_meta(pos)
            if not fields.text then return end
            minetest.log("action", (player_name or "") .. " wrote \"" ..
                fields.text .. "\" to sign at " .. minetest.pos_to_string(pos))
            meta:set_string("text", fields.text)
            meta:set_string("infotext", '"' .. fields.text .. '"')
        end ,
    })
end

-- ===================================================================

register_sign("wood", "Wooden", {
    sounds = default.sound_wood() ,
    groups = {choppy = 2, attached_node = 1, flammable = 2, oddly_breakable_by_hand = 3}
})

-- ===================================================================

register_sign("steel", "Steel", {
    sounds = default.node_sound_defaults() ,
    groups = {cracky = 2, attached_node = 1}
})

-- ===================================================================

minetest.register_node ("default:ladder_wood", {
    description = "Wooden Ladder",
    mesh = "default_ladder.b3d",
    tiles = {"default_ladder_wood.png"} ,
    paramtype = "light",
    paramtype2 = "facedir",
    sunlight_propagates=true,
    walkable=true,
    climbable=true,
    is_ground_content=false,
    groups = {choppy=1.5, oddly_breakable_by_hand=2.5, flammable=2} ,
    drawtype="mesh",
    collision_box = {
        type="fixed",
        fixed = {
            { -0.47, -0.5, 0.335, 0.47, 0.5, 0.5 }
        }
    } ,
    selection_box = {
        type="fixed",
        fixed = {
            {-0.47, -0.5, 0.335, 0.47, 0.5, 0.5}
        }
    } ,

    legacy_wallmounted = true,
    sounds = default.sound_wood()
})

-- ===================================================================

minetest.register_lbm ({
    name      =   "default:cvt_ladder"   ,
    label     =   "default:cvt_ladder"   ,
    nodenames = { "default:ladder_steel" ,
                  "default:ladder_wood"  ,
    } ,
    run_at_every_load = true ,

    action = function (pos, node)
        local meta = minetest.get_meta (pos)
        if meta:get_int ("cvt") ~= 0 then return end
           meta:set_int ("cvt", 1)

        local rotmap = {
            [4] = 0, [2] = 1, [5] = 2, [3] = 3
        }

        local  old_fdir =  node.param2
        local  new_fdir =  rotmap [old_fdir] or old_fdir

-- placed on wall w/player facing north: old=4 new=0
-- placed on wall w/player facing east:  old=2 new=1
-- placed on wall w/player facing south: old=5 new=2
-- placed on wall w/player facing west:  old=3 new=3

        if old_fdir ~= new_fdir then
            node.param2 = new_fdir
            minetest.set_node (pos, node)
        end
    end
})

-- ===================================================================

minetest.register_node ("default:ladder_steel", {
    description = "Steel Ladder",
    mesh = "default_ladder.b3d",
    tiles = {"default_ladder_steel.png"} ,
    paramtype = "light",
    paramtype2 = "facedir",
    sunlight_propagates=true,
    walkable=true,
    climbable=true,
    is_ground_content=false,
    groups = {choppy=1.5, oddly_breakable_by_hand=2.5, flammable=2} ,
    drawtype="mesh",

    collision_box = {
        type="fixed",
        fixed = {
            {-0.47, -0.5, 0.335, 0.47, 0.5, 0.5}
        }
    } ,

    selection_box = {
        type="fixed",
        fixed = {
            {-0.47, -0.5, 0.335, 0.47, 0.5, 0.5}
        }
    } ,

    legacy_wallmounted = true,
    sounds = default.node_sound_metal_defaults()
})

-- ===================================================================

default.register_fence ("default:fence_wood", {
    description = "Wooden Fence",
    texture = "default_fence_wood.png",
    material = "default:wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence ("default:fence_acacia_wood", {
    description = "Acacia Fence",
    texture = "default_fence_acacia_wood.png",
    material = "default:acacia_wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence ("default:fence_aspen_wood", {
    description = "Aspen Fence",
    texture = "default_fence_aspen_wood.png",
    material = "default:aspen_wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence ("default:fence_junglewood", {
    description = "Junglewood Fence",
    texture = "default_fence_junglewood.png",
    material = "default:junglewood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence ("default:fence_maple_wood", {
    description = "Maple Fence",
    texture = "default_fence_maple_wood.png",
    material = "default:aspen_wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence ("default:fence_pine_wood", {
    description = "Pine Fence",
    texture = "default_fence_pine_wood.png",
    material = "default:pine_wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

minetest.register_node ("default:glass", {
    description = "Glass",
    drawtype = "glasslike_framed_optional",
    tiles = {"default_glass.png", "default_glass_detail.png"} ,
    paramtype = "light",
    sunlight_propagates = true,
    is_ground_content = false,
    groups = {cracky = 3, oddly_breakable_by_hand = 3} ,
    sounds = default.node_sound_glass_defaults() ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:clean_super_glow_glass", {
    description = "Clean Super Glow Glass"    ,
    drawtype    = "glasslike_framed_optional" ,

    tiles       = {
        "default_clean_glass.png^[colorize:#FFFF78"        ,
        "default_clean_glass_detail.png^[colorize:#FFFF78" ,
    } ,

    paramtype           = "light"           ,
    sunlight_propagates = true              ,
    is_ground_content   = false             ,
    light_source        = default.LIGHT_MAX ,

    groups = { cracky = 3, oddly_breakable_by_hand = 3 } ,
    sounds = default.node_sound_glass_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:obsidian_glass", {
    description = "Obsidian Glass",
    drawtype = "glasslike_framed_optional",
    tiles = {"default_obsidian_glass.png", "default_obsidian_glass_detail.png"} ,
    paramtype = "light",
    is_ground_content = false,
    sunlight_propagates = true,
    sounds = default.node_sound_glass_defaults() ,
    groups = {cracky = 3} ,
    workbench = true ,
})

-- ===================================================================

minetest.register_node ("default:brick", {
    description = "Brick Block",
    tiles = {"default_brick.png"} ,
    is_ground_content = false,
    groups = { cracky = 3 } ,
    sounds = default.sound_stone() ,
    workbench = true ,
})

-- ===================================================================
-- Misc

minetest.register_node ("default:cloud", {
    description = "Cloud",
    tiles = {"default_cloud.png"} ,
    is_ground_content = false,
    sounds = default.node_sound_defaults() ,
    groups = {not_in_creative_inventory = 1} ,
})

-- ===================================================================

minetest.register_node ("default:nyancat", {
    description = "Nyan Cat",
    tiles = {"default_nc_side.png", "default_nc_side.png", "default_nc_side.png",
        "default_nc_side.png", "default_nc_back.png", "default_nc_front.png"} ,
    paramtype2 = "facedir",
    groups = {cracky = 2} ,
    is_ground_content = false,
    legacy_facedir_simple = true,
    sounds = default.node_sound_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:nyancat_rainbow", {
    description = "Nyan Cat Rainbow",
    tiles = {
        "default_nc_rb.png^[transformR90", "default_nc_rb.png^[transformR90",
        "default_nc_rb.png", "default_nc_rb.png"
    } ,
    paramtype2 = "facedir",
    groups = {cracky = 2} ,
    is_ground_content = false,
    sounds = default.node_sound_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:torch_ceiling", {
    drawtype = "mesh" ,
    mesh     = "torch_ceiling.obj" ,
    tiles    = {{
        name = "default_torch_on_floor_animated.png" ,
        animation = {
            type  = "vertical_frames",
            aspect_w = 16, aspect_h = 16, length = 3.3
        }
    }} ,

    paramtype           = "light"       ,
    paramtype2          = "wallmounted" ,
    sunlight_propagates = true          ,
    walkable            = false         ,
    light_source        = 12            ,

    groups = {
        choppy=2, dig_immediate=3, flammable=1,
        not_in_creative_inventory=1, attached_node=1, torch=1
    } ,

    drop = "default:torch" ,
    selection_box = {
        type = "wallmounted" ,
        wall_top = {-1/8, -1/16, -5/16, 1/8, 1/2, 1/8} ,
    } ,
    sounds = default.sound_wood(),
})

-- ===================================================================

minetest.register_node ("default:torch_wall", {
    drawtype = "mesh"           ,
    mesh     = "torch_wall.obj" ,
    tiles    = {{
        name = "default_torch_on_floor_animated.png" ,
        animation = {
            type  = "vertical_frames" ,
            aspect_w = 16, aspect_h = 16, length = 3.3
        }
    }} ,

    paramtype           = "light"       ,
    paramtype2          = "wallmounted" ,
    sunlight_propagates = true          ,
    walkable            = false         ,
    light_source        = 12            ,

    groups = {
        choppy=2, dig_immediate=3, flammable=1,
        not_in_creative_inventory=1, attached_node=1, torch=1
    } ,

    drop = "default:torch" ,
    selection_box = {
        type = "wallmounted" ,
        wall_side = {-1/2, -1/2, -1/8, -1/8, 1/8, 1/8} ,
    } ,
    sounds = default.sound_wood(),
})
reg_alias ("torches:wall", "default:torch_wall")

-- ===================================================================
-- Coral death near air

minetest.register_abm ({
    nodenames = {
        "default:coral_blue"   ,
        "default:coral_brown"  ,
        "default:coral_orange" ,
    } ,

    neighbors = { "air" } ,
    interval  = 20        ,
    chance    =  5        ,
    catch_up  = false     ,

    action    = function (pos, node)
        minetest.set_node (pos, {name = "default:coral_skeleton"})
    end ,
})

-- =================================================================

minetest.register_node ("default:meselamp", {

    description         = "Mese Lamp"                ,
    drawtype            = "glasslike"                ,
    is_ground_content   = false                      ,
    light_source        = default.LIGHT_MAX - 1      ,
    paramtype           = "light"                    ,
    sunlight_propagates = true                       ,
    tiles               = { "default_meselamp.png" } ,

    groups = { cracky=3, oddly_breakable_by_hand=3 } ,
    sounds = default.node_sound_glass_defaults()     ,
})

-- =================================================================

minetest.register_node ("default:meselamp_white", {

    description         = "Mese Lamp White"     ,
    drawtype            = "glasslike"           ,
    groups              = { cracky=2 }          ,
    is_ground_content   = false                 ,
    light_source        = default.LIGHT_MAX - 1 ,
    paramtype           = "light"               ,
    sunlight_propagates = true                  ,

    sounds = default.node_sound_glass_defaults()          ,
    tiles  = { "jtlight.png^default_obsidian_glass.png" } ,
})

-- =================================================================

default.register_mesepost ("default:mese_post_light", {
    description = "Apple Wood Mese Post Light" ,
    texture     = "default_fence_wood.png"     ,
    material    = "default:wood"               ,
})

-- =================================================================

if false then
minetest.register_node ("default:mese_post_light", {

    description         = "Mese Post Light"     ,
    drawtype            = "nodebox"             ,
    paramtype           = "light"               ,
    light_source        = default.LIGHT_MAX - 1 ,
    sunlight_propagates = true                  ,
    is_ground_content   = false                 ,

    tiles = {
        "default_mese_post_light_top.png"       ,
        "default_mese_post_light_top.png"       ,
        "default_mese_post_light_side_dark.png" ,
        "default_mese_post_light_side_dark.png" ,
        "default_mese_post_light_side.png"      ,
        "default_mese_post_light_side.png"      ,
    } ,

    wield_image = "default_mese_post_light_side.png" ,

    node_box = {
        type  = "fixed" ,
        fixed = {
            { -2/16, -8/16, -2/16, 2/16, 8/16, 2/16 } ,
        } ,
    } ,

    groups = {
        choppy=2, oddly_breakable_by_hand=2, flammable=2
    } ,

    sounds = default.sound_wood() ,
})
end

-- ===================================================================

default.register_mesepost ("default:mese_post_light_acacia_wood", {
    description = "Acacia Wood Mese Post Light"   ,
    texture     = "default_fence_acacia_wood.png" ,
    material    = "default:acacia_wood"           ,
})

-- ===================================================================

default.register_mesepost ("default:mese_post_light_aspen_wood", {
    description = "Aspen Wood Mese Post Light"   ,
    texture     = "default_fence_aspen_wood.png" ,
    material    = "default:aspen_wood"           ,
})

-- ===================================================================

default.register_mesepost ("default:mese_post_light_junglewood", {
    description = "Jungle Wood Mese Post Light"  ,
    texture     = "default_fence_junglewood.png" ,
    material    = "default:jungle_wood"          ,
})

-- ===================================================================

default.register_mesepost ("default:mese_post_light_pine_wood", {
    description = "Pine Wood Mese Post Light"   ,
    texture     = "default_fence_pine_wood.png" ,
    material    = "default:pine_wood"           ,
})

-- ===================================================================

minetest.register_node ("default:permafrost", {

    description = "Permafrost",
    tiles = {"default_permafrost.png"} ,
    groups = {cracky = 3} ,
    sounds = default.node_sound_dirt_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:permafrost_with_stones", {

    description = "Permafrost with Stones",
    tiles = {"default_permafrost.png^default_stones.png",
        "default_permafrost.png",
        "default_permafrost.png^default_stones_side.png"} ,
    groups = {cracky = 3} ,
    sounds = default.node_sound_gravel_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:permafrost_with_moss", {

    description = "Permafrost with Moss",
    tiles = {"default_moss.png", "default_permafrost.png",
        {name = "default_permafrost.png^default_moss_side.png",
            tileable_vertical = false}} ,
    groups = {cracky = 3} ,
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.25} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:blueberry_bush_leaves_with_berries", {

    description = "Blueberry Bush Leaves with Berries",
    drawtype = dtleaves,
    waving = 1,
    tiles = {"default_blueberry_bush_leaves.png^default_blueberry_overlay.png"} ,
    paramtype = "light",
    groups = {snappy = 3, flammable = 2, leaves = 1, dig_immediate = 3} ,
    drop = "default:blueberries",
    sounds = default.sound_leaves() ,
    node_dig_prediction = "default:blueberry_bush_leaves",

    after_dig_node = function (pos, oldnode, oldmetadata, digger)
        minetest.set_node (pos, {name = "default:blueberry_bush_leaves"})
        minetest.get_node_timer(pos):start(math.random(300, 1500))
    end ,
})

-- ===================================================================

minetest.register_node ("default:blueberry_bush_leaves", {

    description = "Blueberry Bush Leaves",
    drawtype = dtleaves,
    waving = 1,
    tiles = {"default_blueberry_bush_leaves.png"} ,
    paramtype = "light",
    groups = {snappy = 3, flammable = 2, leaves = 1} ,
    drop = {
        max_items = 1,
        items = {
            {items = {"default:blueberry_bush_sapling"} , rarity = 5} ,
            {items = {"default:blueberry_bush_leaves"}}
        }
    } ,

    sounds = default.sound_leaves() ,

    on_timer = function (pos, elapsed)
        if minetest.get_node_light(pos) < 11 then
            minetest.get_node_timer(pos):start(200)
        else
            minetest.set_node (pos, {name = "default:blueberry_bush_leaves_with_berries"})
        end
    end ,
})

-- ===================================================================

minetest.register_node ("default:blueberry_bush_sapling", {
    description = "Blueberry Bush Sapling",
    drawtype = "plantlike",
    tiles = {"default_blueberry_bush_sapling.png"} ,
    inventory_image = "default_blueberry_bush_sapling.png",
    wield_image = "default_blueberry_bush_sapling.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
--  on_timer = grow_sapling,

    selection_box = {
        type = "fixed",
        fixed = {-4/16, -0.5, -4/16, 4/16, 2/16, 4/16}
    } ,
    groups = {snappy = 2, dig_immediate = 3, flammable = 2,
        attached_node = 1, sapling = 1} ,
    sounds = default.sound_leaves() ,

--  on_construct = function (pos)
--      minetest.get_node_timer(pos):start(math.random(300, 1500))
--  end ,

    on_place = function (itemstack, placer, pointed_thing)
        itemstack = default.sapling_on_place(itemstack, placer, pointed_thing,
            "default:blueberry_bush_sapling",
            -- minp, maxp to be checked, relative to sapling pos
            {x = -1, y = 0, z = -1} ,
            {x = 1, y = 1, z = 1} ,
            -- maximum interval of interior volume check
            2)

        return itemstack
    end ,
})

default.convert_node (          "default:bush_sapling" ,
                      "default:blueberry_bush_sapling" )

-- ===================================================================

minetest.register_node ("default:pine_bush_stem", {

    description = "Pine Bush Stem",
    drawtype = "plantlike",
    visual_scale = 1.41,
    tiles = {"default_pine_bush_stem.png"} ,
    inventory_image = "default_pine_bush_stem.png",
    wield_image = "default_pine_bush_stem.png",
    paramtype = "light",
    sunlight_propagates = true,
    groups = {choppy = 2, oddly_breakable_by_hand = 1, flammable = 2} ,
    sounds = default.sound_wood() ,

    selection_box = {
        type = "fixed",
        fixed = {-7/16, -0.5, -7/16, 7/16, 0.5, 7/16} ,
    } ,
})

-- ===================================================================

minetest.register_node ("default:pine_bush_needles", {

    description = "Pine Bush Needles",
    drawtype = dtleaves,
    waving = 1,
    tiles = {"default_pine_needles.png"} ,
    paramtype = "light",
    groups = {snappy = 3, flammable = 2, leaves = 1} ,
    drop = {
        max_items = 1,
        items = {
            {items = {"default:pine_bush_sapling"} , rarity = 5} ,
            {items = {"default:pine_bush_needles"}}
        }
    } ,

    sounds = default.sound_leaves() ,
})

-- ===================================================================

minetest.register_node ("default:pine_bush_sapling", {

    description = "Pine Bush Sapling",
    drawtype = "plantlike",
    tiles = {"default_pine_bush_sapling.png"} ,
    inventory_image = "default_pine_bush_sapling.png",
    wield_image = "default_pine_bush_sapling.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
--  on_timer = grow_sapling,

    selection_box = {
        type = "fixed",
        fixed = {-4/16, -0.5, -4/16, 4/16, 2/16, 4/16}
    } ,
    groups = {snappy = 2, dig_immediate = 3, flammable = 2,
        attached_node = 1, sapling = 1} ,
    sounds = default.sound_leaves() ,

--  on_construct = function (pos)
--      minetest.get_node_timer(pos):start(math.random(300, 1500))
--  end ,

    on_place = function (itemstack, placer, pointed_thing)
        itemstack = default.sapling_on_place(itemstack, placer, pointed_thing,
            "default:pine_bush_sapling",
            -- minp, maxp to be checked, relative to sapling pos
            {x = -1, y = 0, z = -1} ,
            {x = 1, y = 1, z = 1} ,
            -- maximum interval of interior volume check
            2)

        return itemstack
    end ,
})

-- ===================================================================

minetest.register_node ("default:dirt_with_rainforest_litter", {

    description = "Dirt with Rainforest Litter",
    tiles = {
        "default_rainforest_litter.png",
        "default_dirt.png",
        {name = "default_dirt.png^default_rainforest_litter_side.png",
            tileable_vertical = false}
    } ,
    groups = {crumbly = 3, soil = 1, spreading_dirt_type = 1} ,
    drop = "default:dirt",
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.4} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:dirt_with_coniferous_litter", {

    description = "Dirt with Coniferous Litter",
    tiles = {
        "default_coniferous_litter.png",
        "default_dirt.png",
        {name = "default_dirt.png^default_coniferous_litter_side.png",
            tileable_vertical = false}
    } ,
    groups = {crumbly = 3, soil = 1, spreading_dirt_type = 1} ,
    drop = "default:dirt",
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.4} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:dry_dirt", {

    description = "Dry Dirt",
    tiles = {"default_dry_dirt.png"} ,
    groups = {crumbly = 3, soil = 1} ,
    sounds = default.node_sound_dirt_defaults() ,
})

-- ===================================================================

minetest.register_node ("default:dry_dirt_with_dry_grass", {

    description = "Dry Dirt with Dry Grass",
    tiles = {"default_dry_grass.png", "default_dry_dirt.png",
        {name = "default_dry_dirt.png^default_dry_grass_side.png",
            tileable_vertical = false}} ,
    groups = {crumbly = 3, soil = 1} ,
    drop = "default:dry_dirt",
    sounds = default.node_sound_dirt_defaults({
        footstep = {name = "default_grass_footstep", gain = 0.4} ,
    }),
})

-- ===================================================================

minetest.register_node ("default:fern_1", {

    description = "Fern",
    drawtype = "plantlike",
    waving = 1,
    tiles = {"default_fern_1.png"} ,
    inventory_image = "default_fern_1.png",
    wield_image = "default_fern_1.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,

    groups = {snappy = 3, flammable = 3, flora = 1, attached_node = 1} ,
    sounds = default.sound_leaves() ,

    selection_box = {
        type = "fixed",
        fixed = {-6/16, -0.5, -6/16, 6/16, -0.25, 6/16} ,
    } ,

    on_place = function (itemstack, placer, pointed_thing)
        -- place a random fern node
        local stack = ItemStack("default:fern_" .. math.random(1, 3))
        local ret = minetest.item_place(stack, placer, pointed_thing)
        return ItemStack("default:fern_1 " ..
            itemstack:get_count() - (1 - ret:get_count()))
    end ,
})

-- ===================================================================

for i = 2, 3 do
    minetest.register_node ("default:fern_" .. i, {
        description = "Fern",
        drawtype = "plantlike",
        waving = 1,
        visual_scale = 2,
        tiles = {"default_fern_" .. i .. ".png"} ,
        inventory_image = "default_fern_" .. i .. ".png",
        wield_image = "default_fern_" .. i .. ".png",
        paramtype = "light",
        sunlight_propagates = true,
        walkable = false,
        buildable_to = true,
        groups = {snappy = 3, flammable = 3, flora = 1, attached_node = 1,
            not_in_creative_inventory=1} ,
        drop = "default:fern_1",
        sounds = default.sound_leaves() ,
        selection_box = {
            type = "fixed",
            fixed = {-6/16, -0.5, -6/16, 6/16, -0.25, 6/16} ,
        } ,
    })
end

-- ===================================================================

minetest.register_node ("default:marram_grass_1", {

    description = "Marram Grass",
    drawtype = "plantlike",
    waving = 1,
    tiles = {"default_marram_grass_1.png"} ,
    inventory_image = "default_marram_grass_1.png",
    wield_image = "default_marram_grass_1.png",
    paramtype = "light",
    sunlight_propagates = true,
    walkable = false,
    buildable_to = true,
    groups = {snappy = 3, flammable = 3, attached_node = 1} ,
    sounds = default.sound_leaves() ,
    selection_box = {
        type = "fixed",
        fixed = {-6/16, -0.5, -6/16, 6/16, -0.25, 6/16} ,
    } ,

    on_place = function (itemstack, placer, pointed_thing)
        -- place a random marram grass node
        local stack = ItemStack("default:marram_grass_" .. math.random(1, 3))
        local ret = minetest.item_place(stack, placer, pointed_thing)
        return ItemStack("default:marram_grass_1 " ..
            itemstack:get_count() - (1 - ret:get_count()))
    end ,
})

-- ===================================================================

for i = 2, 3 do
    minetest.register_node ("default:marram_grass_" .. i, {
        description = "Marram Grass",
        drawtype = "plantlike",
        waving = 1,
        tiles = {"default_marram_grass_" .. i .. ".png"} ,
        inventory_image = "default_marram_grass_" .. i .. ".png",
        wield_image = "default_marram_grass_" .. i .. ".png",
        paramtype = "light",
        sunlight_propagates = true,
        walkable = false,
        buildable_to = true,
        groups = {snappy = 3, flammable = 3, attached_node = 1,
            not_in_creative_inventory=1} ,
        drop = "default:marram_grass_1",
        sounds = default.sound_leaves() ,
        selection_box = {
            type = "fixed",
            fixed = { -6/16, -0.5, -6/16, 6/16, -0.25, 6/16 } ,
        } ,
    })
end

-- ===================================================================

minetest.register_node ("default:rail", {
    description = "Rail"     ,
    drawtype    = "raillike" ,

    tiles       = {
        "default_rail.png"            ,
        "default_rail_curved.png"     ,
        "default_rail_t_junction.png" ,
        "default_rail_crossing.png"   ,
    } ,

    inventory_image   = "default_rail.png" ,
    wield_image       = "default_rail.png" ,
    paramtype         = "light"            ,
    walkable          = false              ,
    is_ground_content = false              ,

    -- but how to specify the dimensions for curved and sideways rails?
    selection_box = {
        type  = "fixed" ,
        fixed = { -1/2,-1/2, -1/2, 1/2, -1/2+1/16, 1/2 } ,
    } ,

    groups = { bendy=2, dig_immediate=2, attached_node=1 } ,
})

-- ===================================================================

default.register_fence_rail ("default:fence_rail_wood", {

    description = "Apple Wood Fence Rail",
    texture = "default_fence_rail_wood.png",
    inventory_image = "default_fence_rail_overlay.png^default_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    wield_image = "default_fence_rail_overlay.png^default_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    material = "default:wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence_rail ("default:fence_rail_acacia_wood", {

    description = "Acacia Wood Fence Rail",
    texture = "default_fence_rail_acacia_wood.png",
    inventory_image = "default_fence_rail_overlay.png^default_acacia_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    wield_image = "default_fence_rail_overlay.png^default_acacia_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    material = "default:acacia_wood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence_rail ("default:fence_rail_junglewood", {

    description = "Jungle Wood Fence Rail",
    texture = "default_fence_rail_junglewood.png",
    inventory_image = "default_fence_rail_overlay.png^default_junglewood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    wield_image = "default_fence_rail_overlay.png^default_junglewood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    material = "default:junglewood",
    groups = {choppy = 2, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence_rail ("default:fence_rail_pine_wood", {

    description = "Pine Wood Fence Rail",
    texture = "default_fence_rail_pine_wood.png",
    inventory_image = "default_fence_rail_overlay.png^default_pine_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    wield_image = "default_fence_rail_overlay.png^default_pine_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    material = "default:pine_wood",
    groups = {choppy = 3, oddly_breakable_by_hand = 2, flammable = 3} ,
    sounds = default.sound_wood()
})

-- ===================================================================

default.register_fence_rail ("default:fence_rail_aspen_wood", {

    description = "Aspen Wood Fence Rail",
    texture = "default_fence_rail_aspen_wood.png",
    inventory_image = "default_fence_rail_overlay.png^default_aspen_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    wield_image = "default_fence_rail_overlay.png^default_aspen_wood.png^" ..
                "default_fence_rail_overlay.png^[makealpha:255,126,126",
    material = "default:aspen_wood",
    groups = {choppy = 3, oddly_breakable_by_hand = 2, flammable = 2} ,
    sounds = default.sound_wood()
})

-- =================================================================

minetest.register_craftitem ("default:bread", {
    description     = "Bread"            ,
    inventory_image = "default_bread.png"   ,
    on_use          = minetest.item_eat (5) ,
    stack_max       = 99                    ,
    groups = { food_bread=1, flammable=2 }  ,
})

-- =================================================================

node_name = "default:glowing_fungal_stone"
--
minetest.register_node (node_name, {
    description       = "Glowing Fungal Stone"  ,
    is_ground_content = true                    ,
    light_source      = 8                       ,
    groups            = { cracky=3, stone=1 }   ,
    sounds            = default.sound_stone()   ,
    paramtype         = "light"                 ,

    tiles = { "default_stone.png^default_glowing_fungal_stone.png" } ,

    drop = {
        items = {
            { items = { "default:cobble"              }} ,
            { items = { "loud_walking:glowing_fungus" }} ,
        }
    } ,
})

-- =================================================================

node_name = "default:glowing_fungus"
img       = "default_glowing_fungus.png"
--
minetest.register_node (node_name, {
    description     = "Glowing Fungus"  ,
    drawtype        = "plantlike"       ,
    paramtype       = "light"           ,
    tiles           = { img }           ,
    inventory_image =   img             ,

    groups          = {
        dig_immediate=3, attached_node=1
    } ,
})

-- =================================================================
-- Mob fence (looks like normal fence but collision is 2 high)

default.register_fence (":mobs:fence_wood", {
    description = "Mob Fence"          ,
    texture     = "default_wood.png"   ,
    material    = "default:fence_wood" ,
    sounds      = default.sound_wood() ,

    groups      = {
        choppy=2, oddly_breakable_by_hand=2, flammable=2,
    } ,

    collision_box = {
        type  = "fixed" ,
        fixed = {
            { -0.5, -0.5, -0.5, 0.5, 1.9, 0.5 } ,
        } ,
    } ,
})

-- ===================================================================
-- Mob fence top (larger collisionbox to keep mobs from getting over)

local mftbox = { -0.4, -1.5, -0.4, 0.4, 0.0, 0.4 }

minetest.register_node (":mobs:fence_top", {
    description       = "Mob Fence Top"         ,
    drawtype          = "nodebox"               ,
    tiles             = { "default_wood.png" }  ,
    paramtype         = "light"                 ,
    is_ground_content = false                   ,
    sounds            = default.sound_wood()    ,

    groups = {
        choppy=2, oddly_breakable_by_hand=2, flammable=2,
    } ,

    node_box = {
        type  = "fixed" ,
        fixed = { -0.2, -0.5, -0.2, 0.2, 0, 0.2 } ,
    } ,

    collision_box = {
        type  = "fixed" ,
        fixed = mftbox  ,
    } ,

    selection_box = {
        type  = "fixed" ,
        fixed = mftbox  ,
    } ,
})

-- ===================================================================
-- End of module.
