-- Denny -- Largely original raptor-type mob.

-- ===================================================================
-- Code license. Applies to this module only.
--
-- CC-BY-NC-SA 3.0.  (c) 2019 and attribution:  OldCoder  (Robert Kir-
-- aly), Poikilos.

-- ===================================================================
-- Media license.
--
-- Model and associated texture.
--
-- You may  copy,  use,  modify or do nearly anything  but remove this
-- copyright notice. Of course,  you're not allowed to pretend  you've
-- created or written the pieces by Christian9 aka Ceej, Poikilos, and
-- OldCoder.
--
-- (c) 2017-2019 and CC BY-SA 3.0:  Christian9 aka Ceej, Poikilos, and
-- OldCoder. If you have additional or contradictory information rela-
-- ted to the history of the model, inform Final Minetest.
--
-- Sounds:
-- CC BY-SA 3.0: Poikilos and:
-- [Mike Koenig](http://soundbible.com/1165-Dinosaur-Roar.html)

-- ===================================================================

local lcname     = "denny"
local ucname     = "Denny"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  60000       ,

    day_mode     = true         ,
    min_light    =      3       ,
    max_light    =     20       ,
    min_height   =    -18       ,
    max_height   =    500       ,

    spawn_type   = "monster"    ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass" ,
    } ,

    add_egg   = true              ,
    egg_image = "wool_orange.png" ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type    ,
    makes_footstep_sound = false    ,
    fall_speed     =  -9.8          ,
    floats         =   0            ,
    fear_height    =   6            ,
    stepheight     = 2.2            ,
    jump_height    =   0            ,
    jump           = false          ,

    armor          = 100            ,
    hp_min         =   7            ,
    hp_max         =  33            ,
    fall_damage    =   0            ,
    lava_damage    =   5            ,
    light_damage   =   0            ,
    water_damage   =   1            ,
    floats = 0,

    passive        = false          ,
    pathfinding    = true           ,
    runaway        = false          ,
    reach          =   2            ,
    attack_animals = true           ,
    attack_type    = "dogfight"     ,
    view_range     =  10            ,
    damage         =   3            ,

    -- at 4 fps, visual_size {x=1,y=1}, walk_velocity is .4 m/s
    -- at 4 fps, visual_size {x=1,y=1}, run_velocity  is .8 m/s

    walk_velocity  =   1.6          ,
    run_velocity   =   3.2          ,

    textures       = {{ msname .. "_mesh.png" }}             ,
    collisionbox   = { -0.33, 0.00, -.33, 0.33, 1.89, 0.33 } ,
    visual         = "mesh"                                  ,
    mesh           = msname .. ".b3d"                        ,
    visual_size    = { x = 4.0, y = 4.0 }                    ,

    animation     = {
        speed_normal =  2 ,
        stand_start  =  0 ,
        stand_end    =  5 ,
        stand_speed  =  2 ,
        walk_start   =  6 ,
        walk_end     = 10 ,
        walk_speed   =  4 ,
        run_start    = 11 ,
        run_end      = 15 ,
        run_speed    =  4 ,
        punch_start  = 16 ,
        punch_end    = 19 ,
        punch_speed  =  4 ,
        punch2_start = 20 ,
        punch2_end   = 25 ,
        punch2_speed =  4 ,
        die_start    = 26 ,
        die_end      = 39 ,
        die_speed    =  8 ,
        die_loop     = false,
    },

    sounds = {
        random = msname ,
        --war_cry = msname .. "_war_cry" ,
        death = msname .. "_death" ,
    },

    drops = {
        { name="mobs:meat_raw", chance=1, min=1, max=1 } ,
    } ,

    -- follow = { "mobs:meat_raw" } ,

    -- on_rightclick = function (self, clicker)
    --     mobs:protect (self, clicker)
    --     mobs:capture_mob (self, clicker, 30, 50, 80, false, nil)
    -- end,
}

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("codermobs:deinonychus" , obj_name)
mobs:alias_mob ("codermobs:raptor"      , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
