-- Moon Cow by Nathan S.

-- ===================================================================

local lcname     = "mooncow"
local ucname     = "Moon Cow"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =      2       ,
    spawn_chance =  12000       ,

    day_mode     = nil          ,
    min_light    =      1       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    scale        = 1.00         ,
    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "moontest:basalt"               ,
        "moontest:dust"                 ,
        "moontest:stone"                ,
    } ,

    add_egg   = true                    ,
    egg_image = "default_grass.png"     ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type         = mobs_param.spawn_type    ,

    passive      = false,
    attack_type  = "dogfight",
    reach        = 2,
    damage       = 4,
    hp_min       = 5,
    hp_max       = 20,
    armor        = 200,
    collisionbox = {-0.4, -0.01, -0.4, 0.4, 1, 0.4},
    visual       = "mesh",
    visual_size = {x = 9, y = 9},
    mesh = "codermobs_mooncow.b3d",

    textures = {
        { "mooncow-white.png" } ,
        { "mooncow-brown.png" } ,
    },

    makes_footstep_sound = true,
    walk_velocity = 1,
    run_velocity = 2,
    stepheight    = 1.2         ,
    jump_height   =   0         ,
    jump          = false       ,

    drops = {
        { name="mobs:meat_raw" , chance=1, min=1, max=3 } ,
        { name="mobs:leather"  , chance=1, min=1, max=2 } ,
    } ,

    water_damage = 1,
    lava_damage = 5,
    light_damage = 0,
    animation = {
        speed_normal = 20,
        speed_run = 30,
        stand_start = 42,
        stand_end = 82,
        walk_start = 0,
        walk_end = 40,
        run_start = 0,
        run_end = 40,
        punch_start = 70,
        punch_end = 100,
    },
    follow = "farming:wheat",
    view_range = 7,

    fear_height = 2,
    on_rightclick = function(self, clicker)
        -- feed or tame
        if mobs:feed_tame(self, clicker, 8, true, true) then
            return
        end

        local tool = clicker:get_wielded_item()

        -- milk cow with empty bucket
        if tool:get_name() == "bucket:bucket_empty" then

            --if self.gotten == true
            if self.child == true then
                return
            end

            if self.gotten == true then
                minetest.chat_send_player(clicker:get_player_name(),
                        "Cow already milked!")
                return
            end

            local inv = clicker:get_inventory()

            inv:remove_item("main", "bucket:bucket_empty")

            if inv:room_for_item("main", {name = "mobs:bucket_milk"}) then
                clicker:get_inventory():add_item("main", "mobs:bucket_milk")
            else
                local pos = self.object:getpos()
                pos.y = pos.y + 0.5
                minetest.add_item(pos, {name = "mobs:bucket_milk"})
            end

            self.gotten = true -- milked

            return
        end

        mobs:capture_mob(self, clicker, 0, 5, 60, false, nil)
    end,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
