-- Rooster. Descended from "Farlands" version.

-- ===================================================================
-- License information for Sapier's 3D-egg media.
--
-- You may  copy,  use,  modify or do nearly anything  but remove this
-- copyright notice. Of course,  you're not allowed to pretend  you've
-- created or written the Sapier pieces.
--
-- CC-BY-SA 3.0. Attribution: Sapier.

-- ===================================================================

local lcname      = "chicken"
local ucname      = "Chicken"
local msname      = "codermobs_" .. lcname
local obj_name    = "codermobs:" .. lcname

-- This mob is a special case
local obj_rooster = "codermobs:rooster"

-- ===================================================================

mobs_param = {
    lcname       = "rooster"           ,
    ucname       = "Rooster"           ,
    obj_name     = "codermobs:rooster" ,

    aoc          =      1       ,
    spawn_chance =  20000       ,

    day_mode     = true         ,
    min_light    =      5       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass"   ,
        "default:dirt_with_grass"       ,
        "earthgen:dirt_with_dry_grass"  ,
        "ethereal:bamboo_dirt"          ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "ethereal:grove_dirt"           ,
        "mg:dirt_with_dry_grass"        ,
        "noisegrid:grass"               ,
    } ,

    add_egg   = true                    ,
    egg_image = "wool_brown.png"        ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

local obj_name_raw          = obj_name         .. "_raw"

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = true  ,

    -- Most behavior/sight/battle
    armor          = 200         ,
    attack_type    = "dogfight"  ,
    damage         =   1         ,
    hp_max         =  10         ,
    hp_min         =   5         ,
    passive        = false       ,
    reach          =   1         ,
    runaway        = true        ,
    view_range     =   5         ,

    -- Velocities
    run_velocity   =   3         ,
    walk_velocity  =   1         ,

    -- Step/jump/fall parameters
    fall_damage    =   0         ,
    fall_speed     =  -8         ,
    fear_height    =   5         ,
    jump           = false       ,
    jump_height    =   0         ,
    stepheight     =   1.2       ,

    -- Environment damage
    lava_damage    =   5         ,
    light_damage   =   0         ,
    water_damage   =   1         ,

    -- Misc. parameters
    floats         =   0         ,

    collisionbox     = { -0.30, 0.00, -0.30, 0.30, 0.67, 0.30 } ,
    mesh             = msname .. ".b3d"                         ,
    textures         = {{ "codermobs_rooster_mesh.png" }}       ,
    visual           = "mesh"                                   ,
    visual_size      = { x=1.5, y=1.5 }                         ,

    animation        = {
        stand_start  =  0 ,
        stand_end    = 20 ,
        walk_start   = 25 ,
        walk_end     = 45 ,
        speed_normal = 15 ,
    } ,

    drops            = {
        { name=obj_name_raw, chance=1, min=2, max=2 } ,
    } ,

    follow           = {
        "farming:seed_wheat", "farming:seed_cotton"
    } ,

    sounds           = { random = msname } ,

    specific_attack  = codermobs.make_mob_list ({
        "bug", "beetle"
    }) ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame (self, clicker, 8, true, true) then
            return
        end
        mobs:protect (self, clicker)
        mobs:capture_mob (self, clicker, 30, 50, 80, false, nil)
    end,

    on_breed = function (self, ent)
        return false
    end ,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
