-- Sheep. Descended from Farlands version.

-- ===================================================================

local lcname    = "sheep"
local ucname    = "Sheep"
local msname    = "codermobs_" .. lcname

-- This set  of mobs  is a  special case.  Structure of mob definition
-- file is different.

-- ===================================================================

local codermobs_disable_monsters       =
    minetest.setting_getbool ("codermobs_disable_monsters"      )

local codermobs_enable_thismob         =
    codermobs.mob_enabled (lcname)

local codermobs_logless                =
    minetest.setting_getbool ("codermobs_logless"               )

local codermobs_monster_global         =
    minetest.setting_getbool ("codermobs_monster_global"        )

local codermobs_monster_thismob        =
    minetest.setting_getbool ("codermobs_monster_"  .. lcname   )

local codermobs_peaceful_global        =
    minetest.setting_getbool ("codermobs_peaceful_global"       )

local codermobs_peaceful_thismob       =
    minetest.setting_getbool ("codermobs_peaceful_" .. lcname   )

-- ===================================================================

local aoc          =     -1
local daymode      = true
local minlight     =     10
local maxlight     =     30
local minheight    =      0
local maxheight    =  31000

local spawn_chance = 450000
local spawn_type   = "animal"

local spawn_nodes  = {
    "default:dirt_with_dry_grass"   ,
    "default:dirt_with_grass"       ,
    "ethereal:bamboo_dirt"          ,
    "fargen:dirt_with_leafy_grass"  ,
    "noisegrid:grass"               ,
}

-- ===================================================================

if (spawn_type == "monster") or (spawn_type == "animal") then
    if codermobs_monster_global   or  codermobs_monster_thismob  then
        spawn_type = "monster"
    end
    if codermobs_peaceful_global  or  codermobs_peaceful_thismob then
        spawn_type = "animal"
    end
    if spawn_type == "monster" and codermobs_disable_monsters then
        codermobs_enable_thismob = false
    end
end

-- ===================================================================

local xyz

local function getnumsetting (name)
    local  n = tonumber (minetest.setting_get (name)) or 0
    return n
end

-- ===================================================================

    xyz = getnumsetting ("codermobs_minheight_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_minheight_global")
end
if  xyz ~= 0 then
    minheight = xyz
end
    xyz = getnumsetting ("codermobs_maxheight_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_maxheight_global")
end
if  xyz ~= 0 then
    maxheight = xyz
end

-- ===================================================================

    xyz = getnumsetting ("codermobs_minlight_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_minlight_global")
end
if  xyz ~= 0 then
    minlight = xyz
end
    xyz = getnumsetting ("codermobs_maxlight_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_maxlight_global")
end
if  xyz ~= 0 then
    maxlight = xyz
end

-- ===================================================================

    xyz = getnumsetting ("codermobs_aoc_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_aoc_global")
end
if  xyz ~= 0 then
    aoc = xyz
end

-- ===================================================================

    xyz = getnumsetting ("codermobs_chance_" .. lcname)
if  xyz == 0 then
    xyz = getnumsetting ("codermobs_chance_global")
end
if  xyz ~= 0 then
    spawn_chance = xyz
end

-- ===================================================================

local all_colours = {
    {"black",      "Black",      "#000000b0"},
    {"blue",       "Blue",       "#015dbb70"},
    {"brown",      "Brown",      "#663300a0"},
    {"cyan",       "Cyan",       "#01ffd870"},
    {"dark_green", "Dark Green", "#005b0770"},
    {"dark_grey",  "Dark Grey",  "#303030b0"},
    {"green",      "Green",      "#61ff0170"},
    {"grey",       "Grey",       "#5b5b5bb0"},
    {"magenta",    "Magenta",    "#ff05bb70"},
    {"orange",     "Orange",     "#ff840170"},
    {"pink",       "Pink",       "#ff65b570"},
    {"red",        "Red",        "#ff0000a0"},
    {"violet",     "Violet",     "#2000c970"},
    {"white",      "White",      "#abababc0"},
    {"yellow",     "Yellow",     "#e3ff0070"},
}

-- Sheep by PilzAdam, taken from mobs_animal

for _, col in pairs (all_colours) do
    local col_name = lcname .. "_" .. col [1]
    local obj_name = "codermobs:"  .. col_name

    mobs:register_mob (obj_name, {
        type = spawn_type ,
        passive = true,
        hp_min = 8,
        hp_max = 10,
        armor = 200,
        collisionbox = {-0.4, 0, -0.4, 0.4, 1.3, 0.4},
        visual = "mesh",
        mesh = msname .. ".b3d",

        textures = {
            {
msname .. "_base.png^" ..
msname .. "_" .. col [1] .. ".png"
            } ,
        },

        gotten_texture = { msname .. "_shaved.png"},
        gotten_mesh = msname .. ".b3d",
        makes_footstep_sound = true,
        sounds = {
            random = msname ,
        },

        walk_velocity = 1,
        run_velocity = 2,
        runaway = true,

        stepheight    = 1.2         ,
        jump_height   =   0         ,
        jump          = false       ,

        drops = {
            { name="mobs:meat_raw", chance=1, min=1, max=2 } ,

--{name = "wool:"..col[1], chance = 1, min = 1, max = 1},
        } ,

        water_damage = 1,
        floats = 0,
        runaway = true,
        lava_damage = 5,
        light_damage = 0,

        animation = {
            stand_start  =  1 ,
            stand_end    = 25 ,
            walk_start   = 27 ,
            walk_end     = 46 ,
            run_start    = 27 ,
            run_end      = 46 ,
            speed_normal = 16 ,
            speed_run    = 20 ,
        } ,

        follow = {"farming:wheat", "default:grass_5"},
        view_range = 8,
        replace_rate = 10,
        replace_what = {"default:grass_3", "default:grass_4", "default:grass_5", "farming:wheat_8"},
        replace_with = "air",
        replace_offset = -1,
        fear_height = 3,

        on_rightclick = function(self, clicker)

            --are we feeding?
            if mobs:feed_tame(self, clicker, 8, true, true) then

                --if full grow fuzz
                if self.gotten == false then

                    self.object:set_properties({
                        textures = {
msname .. "_base.png^" ..
msname .. "_" .. col [1] .. ".png"
                        } ,
                        mesh = msname .. ".b3d",
                    })
                end

                return
            end

            local item = clicker:get_wielded_item()
            local itemname = item:get_name()

            --are we giving a haircut>
            if itemname == "mobs:shears" then

                if self.gotten ~= false
                or self.child ~= false
                or not minetest.get_modpath("wool") then
                    return
                end

                self.gotten = true -- shaved

                local obj = minetest.add_item(
                    self.object:getpos(),
                    ItemStack( "wool:" .. col[1] .. " " .. math.random(1, 3) )
                )

                if obj then

                    obj:setvelocity({
                        x = math.random(-1, 1),
                        y = 5,
                        z = math.random(-1, 1)
                    })
                end

                item:add_wear(650) -- 100 uses

                clicker:set_wielded_item(item)

                self.object:set_properties({
                    textures = { msname .. "_shaved.png"},
                    mesh = msname .. ".b3d",
                })

                return
            end

            local name = clicker:get_player_name()

            --are we coloring?
            if itemname:find("dye:") then

                if self.gotten == false
                and self.child == false
                and self.tamed == true
                and name == self.owner then

                    local colr = string.split(itemname, ":")[2]

                    for _,c in pairs(all_colours) do

                        if c[1] == colr then

                            local pos = self.object:getpos()

                            self.object:remove()

                            local mob = minetest.add_entity(pos,
msname .. "_" .. colr)
                            local ent = mob:get_luaentity()

                            ent.owner = name
                            ent.tamed = true

                            -- take item
                            if not minetest.setting_getbool("creative_mode") then
                                item:take_item()
                                clicker:set_wielded_item(item)
                            end

                            break
                        end
                    end
                end

                return
            end

            --are we capturing?
            mobs:capture_mob(self, clicker, 0, 5, 60, false, nil)
        end
    })

    mobs:register_egg (obj_name,
        col [2] .. " Sheep", "wool_" .. col [1] .. ".png", true)

    mobs:alias_mob ("mobs:"         .. col_name, obj_name)
    mobs:alias_mob ("mobs_animals:" .. col_name, obj_name)

    if not codermobs_disable_all and
       codermobs_enable_thismob   then
        mobs:register_spawn (obj_name, spawn_nodes,
            maxlight, minlight, spawn_chance, aoc, maxheight, daymode)
    end
end
