-- Tree Monster.

-- ===================================================================

local lcname     = "tree_monster"
local ucname     = "Tree Monster"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

local box_deadster = {
    type  = "fixed" ,
    fixed = { -0.4, -1, -0.4, 0.4, 0.8, 0.4 } ,
}

minetest.register_node ("codermobs:tree_deadster", {
    description     = "Dead Tree Monster"      ,
    drawtype        = "mesh"                   ,
    inventory_image = "default_tree_top.png"   ,
    wield_image     = "default_tree_top.png"   ,
    paramtype       = "light"                  ,
    paramtype2      = "facedir"                ,
    visual_scale    = 0.10                     ,

    collisionbox    = box_deadster             ,
    selectionbox    = box_deadster             ,

    mesh   =   "codermobs_tree_monster.b3d"          ,
    tiles  = { "codermobs_tree_deadster2.png" }      ,
    groups = { choppy=1, oddly_breakable_by_hand=1 } ,
    sounds = default.node_sound_wood_defaults()      ,
})

minetest.register_craft({
    output = 'default:wood 4',
    recipe = {
        {'codermobs:tree_deadster'},
    }
})

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  25000       ,

    day_mode     = false        ,
    min_light    =      0       ,
    max_light    =      7       ,
    min_height   =      0       ,
    max_height   =     75       ,

    spawn_type   = "monster"    ,

    spawn_nodes  = {
        "default:leaves"               ,
        "default:jungleleaves"         ,
        "default:pine_needles"         ,
    } ,

    add_egg   = true                   ,
    egg_image = "default_tree_top.png" ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,

    armor          = 100            ,
    attack_animals = true           ,
    attack_type    = "dogfight"     ,
    damage         =   2            ,
    floats         =   0            ,
    hp_min         =   7            ,
    hp_max         =  33            ,
    stepheight     = 1.2            ,
    jump_height    =   0            ,
    jump           = false          ,
    passive        = false          ,
    pathfinding    = true           ,
    reach          =   2            ,
    run_velocity   =   3            ,
    runaway        = true           ,
    view_range     =  15            ,
    walk_velocity  =   1            ,

    fall_damage    =   0            ,
    lava_damage    =   0            ,
    light_damage   =   2            ,
    water_damage   =   1            ,
    light_flames   = true           ,

    makes_footstep_sound = true     ,

    drops = {
        { name = "default:sapling"        ,
          chance = 3, min = 1, max = 2 }  ,
        { name = "default:junglesapling"  ,
          chance = 3, min = 1, max = 2 }  ,
        { name = "default:apple"          ,
          chance = 2, min = 1, max = 3 }  ,
    } ,

    on_die = function (self, pos)
        minetest.chat_send_all ("<tree_monster> where is my sunscreen")
        minetest.set_node (pos, { name = "codermobs:tree_deadster" })
        self.object:remove()
    end ,

    sounds = {
        random = "codermobs_tree_monster" ,
    } ,

    animation = {
        stand_start  =  0 ,
        stand_end    = 24 ,
        walk_start   = 25 ,
        walk_end     = 47 ,
        punch_start  = 48 ,
        punch_end    = 62 ,
        run_start    = 48 ,
        run_end      = 62 ,
        speed_normal = 15 ,
        speed_run    = 15 ,
    } ,

    textures = {
        { "codermobs_tree_monster1.png" } ,
        { "codermobs_tree_monster2.png" } ,
    },

    blood_texture = "default_wood.png"                ,
    collisionbox  = { -0.4, -1, -0.4, 0.4, 0.8, 0.4 } ,
    mesh          = "codermobs_tree_monster.b3d"      ,
    visual        = "mesh"                            ,
}

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("mobs:tree_monster"         , obj_name)
mobs:alias_mob ("mobs_monster:tree_monster" , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
